# -*- coding: utf-8 -*-
# Folder Color 0.0.52
# Copyright (C) 2012-2015 Marcos Alvarez Costales https://launchpad.net/~costales
#
# Folder Color is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
# 
# Folder Color is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Folder Color; if not, see http://www.gnu.org/licenses
# for more information.

import os, urllib, gettext, glob, subprocess, webbrowser, operator
from gi.repository import Nautilus, Gtk, GObject, Gio, GLib

# i18n
gettext.textdomain('folder-color-common')
_ = gettext.gettext


class FolderColor:
    """Theme/icon properties"""
    def __init__(self, remove_pid=True):
        self.DEFAULT_PATH_THEME = ''
        
        self.PLACES  = 'places'
        self.EMBLEMS = 'emblems'
        self.CUSTOM  = 'custom'
        
        self.FOLDER      = 'folder'
        self.PRE_PLACES  = '_'.join([self.FOLDER, 'color'])
        self.PRE_EMBLEMS = '_'.join([self.FOLDER, 'emblem'])
        self.PRE_CUSTOMS = '_'.join([self.FOLDER, 'custom'])
        
        self.SEARCH_PLACES  = '_'.join([self.PRE_PLACES,  '*.svg'])
        self.SEARCH_EMBLEMS = '_'.join([self.PRE_EMBLEMS, '*.svg'])
        self.SEARCH_CUSTOMS = ''.join([self.PRE_CUSTOMS, '*.svg'])
        
        self.CHECK_PLACE  = 'folder_color_blue'
        self.CHECK_EMBLEM = 'folder_emblem_favorite'
        self.CHECK_CUSTOM = 'folder_custom'
        
        self.DESKTOP   = 'desktop'
        self.DOCUMENTS = 'documents'
        self.DOWNLOADS = 'downloads'
        self.MUSIC     = 'music'
        self.PICTURES  = 'pictures'
        self.PUBLIC    = 'public'
        self.TEMPLATES = 'templates'
        self.VIDEOS    = 'videos'
        
        self.DEFAULT_ICONS   = ('undo', 'edit', 'picker', self.DESKTOP, self.DOCUMENTS, self.DOWNLOADS, self.MUSIC, self.PICTURES, self.PUBLIC, self.TEMPLATES, self.VIDEOS)
        self.DEFAULT_FOLDERS = self._get_default_folders()
        
        self.CFG_PATH        = os.path.join(os.getenv('HOME'), '.config', 'folder-color')
        self.CFG_PATH_CUSTOM = os.path.join(self.CFG_PATH, 'custom_icons')
        self.HIDE_DONATION   = os.path.join(self.CFG_PATH, 'hide_donation')
        self.URL_DONATE      = 'http://gufw.org/donate_foldercolor'
        
        self.GRADIENT_MIDDLE_RANGE = 2500
        self.GRADIENT_DARK_RANGE   = 5000
        self.VALUE_LIGHT  = 'value_light'
        self.VALUE_MIDDLE = 'value_middle'
        self.VALUE_DARK   = 'value_dark'
    
    def get_iconname(self, folder, name, what='places'):
        """For 'green' returns folder_color_green_downloads"""
        if what == self.PLACES:
            prefix = self.PRE_PLACES
        elif what == self.EMBLEMS:
            prefix = self.PRE_EMBLEMS
        elif what == self.CUSTOM:
            prefix = self.FOLDER
        else:
            return ''
        
        if folder in self.DEFAULT_FOLDERS:
            iconname = '_'.join([prefix, name.replace(' ', '_').lower(), self.DEFAULT_FOLDERS[folder]])
            filename = ''.join([self.get_path(what), iconname, '.svg'])
            if os.path.isfile(filename): # Exist?
                return iconname
        
        iconname = '_'.join([prefix, name.replace(' ', '_').lower()])
        return iconname
    
    def get_filename(self, icon, extended=True):
        """From a complete path, returns green_downloads | folder_color_green_downloads | folder_color_green_downloads.svg"""
        icon_name = ''.join(icon.split('/')[-1:]).replace('.svg', '')
        if not extended:
            icon_name = icon_name.replace(self.PRE_PLACES, '').replace(self.PRE_EMBLEMS, '').replace('_', ' ').strip()
        return icon_name
    
    def get_path(self, what='places'):
        """Get current path. By example: /usr/share/icons/hicolor/48x48/places/"""
        icon = Gtk.IconTheme.get_default()
        
        if what == self.PLACES:
            icon_path = icon.lookup_icon(self.CHECK_PLACE,  48, 0)
        elif what == self.EMBLEMS:
            icon_path = icon.lookup_icon(self.CHECK_EMBLEM, 24, 0)
        elif what == self.CUSTOM:
            icon_path = icon.lookup_icon(self.CHECK_CUSTOM, 48, 0)
        else:
            return ''
        
        try:
            path = icon_path.get_filename()
        except:
            return ''
        
        filename = ''.join(path.split('/')[-1:])
        return path.replace(filename, '')
    
    def get_available_icons(self, what='places'):
        """Returns: '/usr/share/icons/hicolor', {'blue':'Azul', 'yellow':'Amarillo'}"""
        path = ''
        names_sorted = {}
        first_iter = True
        
        if what == self.PLACES:
            path = ''.join([self.get_path(what), self.SEARCH_PLACES])
        elif what == self.EMBLEMS:
            path = ''.join([self.get_path(what), self.SEARCH_EMBLEMS])
        elif what == self.CUSTOM:
            path = ''.join([self.get_path(what), self.SEARCH_CUSTOMS])
        else:
            return available_items
        
        available_icons = glob.glob(path)
        
        for icon in available_icons:
            if any(not_count in icon for not_count in self.DEFAULT_ICONS):
                continue
            if what != self.CUSTOM and self.CUSTOM in icon:
                continue
            
            if first_iter: # Get default path
                first_iter = False
                path = '/'.join(icon.split('/')[:5])
            
            # Get icons names and its translations
            color = self.get_filename(icon, False)
            if color:
                names_sorted[color] = _(color.title()) # blue = Blue
        
        names = sorted(names_sorted.items(), key=operator.itemgetter(1))
        return path, names
    
    def _reload_icon(self, folder):
        """Reload the current folder icon"""
        cmd = ['touch', folder]
        proc = subprocess.Popen(cmd, shell=False)
        proc.communicate()
    
    def _get_default_folders(self):
        """Default user folders"""
        folders = {}
        folders[GLib.get_user_special_dir(GLib.USER_DIRECTORY_DESKTOP)]      = self.DESKTOP  
        folders[GLib.get_user_special_dir(GLib.USER_DIRECTORY_DOCUMENTS)]    = self.DOCUMENTS
        folders[GLib.get_user_special_dir(GLib.USER_DIRECTORY_DOWNLOAD)]     = self.DOWNLOADS
        folders[GLib.get_user_special_dir(GLib.USER_DIRECTORY_MUSIC)]        = self.MUSIC    
        folders[GLib.get_user_special_dir(GLib.USER_DIRECTORY_PICTURES)]     = self.PICTURES 
        folders[GLib.get_user_special_dir(GLib.USER_DIRECTORY_PUBLIC_SHARE)] = self.PUBLIC   
        folders[GLib.get_user_special_dir(GLib.USER_DIRECTORY_TEMPLATES)]    = self.TEMPLATES
        folders[GLib.get_user_special_dir(GLib.USER_DIRECTORY_VIDEOS)]       = self.VIDEOS   
        return folders
    
    def _unset_attrib(self, folder_name):
        """Restore to default"""
        folder = Gio.File.new_for_path(folder_name)
        info = folder.query_info('metadata::custom-icon-name', 0, None)
        info.set_attribute('metadata::custom-icon',      Gio.FileAttributeType.INVALID, 0)
        info.set_attribute('metadata::custom-icon-name', Gio.FileAttributeType.INVALID, 0)
        return folder, info
    
    def _write_attrib(self, folder, info):
        """Write attributes to folder"""
        folder.set_attributes_from_info(info, 0, None)
    
    def restore_icon(self, folder_name):
        """Restore by default"""
        folder, info = self._unset_attrib(folder_name)
        self.set_emblem(folder_name, '')
        self._write_attrib(folder, info)
        self._reload_icon(folder_name)
    
    def set_icon(self, folder_name, color):
        """Set color to a folder"""
        folder, info = self._unset_attrib(folder_name) # Restore because I don't know if I set previously with custom-icon or custom-icon-name
        info.set_attribute_string("metadata::custom-icon-name", color)
        self._write_attrib(folder, info)
        self._reload_icon(folder_name)
    
    def set_icon_custom(self, folder_name, src_file, dst_file, custom_colors):
        """Set a custom color to a folder"""
        folder, info = self._unset_attrib(folder_name) # Restore because I don't know if I set previously with custom-icon or custom-icon-name
        self._generate_custom_file(src_file, dst_file, custom_colors)
        info.set_attribute_string('metadata::custom-icon', ''.join(['file://', dst_file]))
        self._write_attrib(folder, info)
        self._reload_icon(folder_name)
    
    def set_emblem(self, folder, emblem=''):
        """Set an emblem"""
        # TODO: Use Gio
        if emblem:
            cmd = ['gvfs-set-attribute', '-t', 'stringv', folder, 'metadata::emblems', emblem]
        else:
            cmd = ['gvfs-set-attribute', '-t', 'unset',   folder, 'metadata::emblems']
        proc = subprocess.Popen(cmd, shell=False)
        proc.communicate()
        
        self._reload_icon(folder)
    
    def custom_color_dialog(self):
        """Shows the dialog and return the color"""
        colorselectiondialog = Gtk.ColorSelectionDialog(_("Choose a Folder Color"))
        selector = colorselectiondialog.get_color_selection()
        response = colorselectiondialog.run()
        colorselectiondialog.destroy()
        
        if response == Gtk.ResponseType.OK:
            color = selector.get_current_color()
            # Control max color range
            if color.red <= self.GRADIENT_DARK_RANGE:
                color.red = self.GRADIENT_DARK_RANGE
            if color.green <= self.GRADIENT_DARK_RANGE:
                color.green = self.GRADIENT_DARK_RANGE
            if color.blue <= self.GRADIENT_DARK_RANGE:
                color.blue = self.GRADIENT_DARK_RANGE
            # Convert to hex
            rgb_ligth = (color.red, color.green, color.blue)
            rgb_middle = (color.red-self.GRADIENT_MIDDLE_RANGE, color.green-self.GRADIENT_MIDDLE_RANGE, color.blue-self.GRADIENT_MIDDLE_RANGE)
            rgb_dark = (color.red-self.GRADIENT_DARK_RANGE, color.green-self.GRADIENT_DARK_RANGE, color.blue-self.GRADIENT_DARK_RANGE)
            htmlcolor_light =  ''.join((str(hex(i/257))[2:].rjust(2, '0') for i in rgb_ligth)).upper()
            htmlcolor_middle = ''.join((str(hex(i/257))[2:].rjust(2, '0') for i in rgb_middle)).upper()
            htmlcolor_dark =   ''.join((str(hex(i/257))[2:].rjust(2, '0') for i in rgb_dark)).upper()
            return (htmlcolor_light, htmlcolor_middle, htmlcolor_dark)
        return None
    
    def _generate_custom_file(self, src_file, dst_file, colors):
        """Parsing template to a custom color"""
        f_input  = open(src_file, 'r') # This has to exists always
        f_output = open(dst_file, 'w')
        
        f_generate  = f_input.read().replace(self.VALUE_LIGHT, colors[0]).replace(self.VALUE_MIDDLE, colors[1]).replace(self.VALUE_DARK, colors[2])
        f_output.write(f_generate)


class FolderColorMenu(GObject.GObject, Nautilus.MenuProvider):
    """Folder Color Menu"""
    def __init__(self):
        self.foldercolor = FolderColor()
        
        self.DEFAULT_PATH_THEME = ''
        dummy_i18n = (_("Brown"), _("Blue"), _("Green"), _("Grey"), _("Orange"), _("Pink"), _("Red"), _("Purple"), _("Yellow"), _("Favorite"), _("Finished"), _("Important"), _("In Progress")) # Need i18n, because colors/emblems are dynamic now
        
        if not os.path.exists(self.foldercolor.CFG_PATH_CUSTOM):
            try:
                os.makedirs(self.foldercolor.CFG_PATH_CUSTOM)
            except OSError as exception:
                pass
            except:
                pass
        pass
    
    def get_file_items(self, window, items):
        """Nautilus invoke this function in its startup > Then, create menu entry"""
        # Checks
        if not self._check_generate_menu(items):
            return
        # Return menu
        return self._generate_menu(items)
    
    def _check_generate_menu(self, items):
        """Show the menu?"""
        # No items selected
        if not len(items):
            return False
        
        # Are there Folder Color icons available?
        if not len(self.foldercolor.get_available_icons('places')[1]):
            return False
        
        # Check only folders
        for item in items:
            # GNOME can only handle files
            if item.get_uri_scheme() != 'file':
                return False
            # Only folders
            if not item.is_directory():
                return False
        
        # All OK > Generate menu
        return True
    
    def _generate_menu(self, items):
        """Generate menu: Color | Custom? | Restore | Emblems? | Preferences"""
        # Main menu [1 or +1 folder(s)]
        if len(items) > 1:
            top_menuitem = Nautilus.MenuItem(name='ChangeFolderColorMenu::Top', label=_("Folders' Color"), icon='folder_color_picker')
        else:
            top_menuitem = Nautilus.MenuItem(name='ChangeFolderColorMenu::Top', label=_("Folder's Color"),  icon='folder_color_picker')
        submenu = Nautilus.Menu()
        top_menuitem.set_submenu(submenu)
        
        # Colors
        self.DEFAULT_PATH_THEME,colors = self.foldercolor.get_available_icons()
        for color in colors:
            name = ''.join(['ChangeFolderColorMenu::Colors"', color[0], '"'])
            item = Nautilus.MenuItem(name=name, label=color[1], icon=self.foldercolor.get_iconname('None', color[0]))
            item.connect('activate', self._menu_activate_color, color[0], items)
            submenu.append_item(item)
        
        # Custom color
        path_theme,customs = self.foldercolor.get_available_icons('custom')
        if len(customs):
            if path_theme == self.DEFAULT_PATH_THEME: # Show custom only if it exists in the current theme
                item_custom = Nautilus.MenuItem(name=''.join(['ChangeFolderColorMenu::"custom"']), label=_("Custom"), icon='folder_color_edit')
                item_custom.connect('activate', self._menu_activate_custom, items)
                submenu.append_item(item_custom)
        
        # Emblems
        path_theme,emblems = self.foldercolor.get_available_icons('emblems')
        for i,emblem in enumerate(emblems):
            if not i: # Separator
                if path_theme != self.DEFAULT_PATH_THEME: # Show emblems only if them exist in the current theme
                    break
                item_sep = Nautilus.MenuItem(name='ChangeFolderColorMenu::Sep1', label=_("Categorize:"), sensitive=False)
                submenu.append_item(item_sep)
            
            name = ''.join(['ChangeFolderEmblemMenu::Emblem"', emblem[0], '"'])
            item = Nautilus.MenuItem(name=name, label=emblem[1], icon=self.foldercolor.get_iconname('None', emblem[0], 'emblems'))
            item.connect('activate', self._menu_activate_emblem, emblem[0], items)
            submenu.append_item(item)
        
        # Restore
        item_sep = Nautilus.MenuItem(name='ChangeFolderEmblemMenu::Sep', label=_("Restore:"), sensitive=False)
        submenu.append_item(item_sep)
        
        item_restore = Nautilus.MenuItem(name='ChangeFolderColorMenu::Restore', label=_("By Default"), icon='folder_color_undo')
        item_restore.connect('activate', self._menu_activate_restore, items)
        submenu.append_item(item_restore)
        
        # Donation
        if not os.path.exists(self.foldercolor.HIDE_DONATION):
            # Separator
            item_sep = Nautilus.MenuItem(name='ChangeFolderColorMenu::Sep2', label=_("Support this app:"), sensitive=False)
            submenu.append_item(item_sep)
            # Donation
            item = Nautilus.MenuItem(name=''.join(['ChangeFolderColorMenu::"donate"']), label=_("Donate? Click for hiding"))
            item.connect('activate', self._menu_activate_donate)
            submenu.append_item(item)
        
        return top_menuitem,
        
    def _menu_activate_color(self, menu, color, folders):
        """Set Custom Color"""
        for each_folder in folders:
            if each_folder.is_gone():
                continue
            
            folder_name = urllib.unquote(each_folder.get_uri()[7:])
            new_color = self.foldercolor.get_iconname(folder_name, color)
            
            self.foldercolor.set_icon(folder_name, new_color)
    
    def _menu_activate_custom(self, menu, folders):
        """Set Color"""
        # Custom
        custom_colors = None
        custom_colors = self.foldercolor.custom_color_dialog()
        if custom_colors == None: # Cancel?
            return
        
        for each_folder in folders:
            if each_folder.is_gone():
                continue
            
            folder_name = urllib.unquote(each_folder.get_uri()[7:])
            src_file = ''.join([self.foldercolor.get_path('custom'), self.foldercolor.get_iconname(folder_name, 'custom', 'custom'), '.svg'])
            dst_file = ''.join([self.foldercolor.CFG_PATH_CUSTOM, '/', self.foldercolor.get_iconname(folder_name, 'custom', 'custom'), '_', custom_colors[0], '.svg'])
            
            self.foldercolor.set_icon_custom(folder_name, src_file, dst_file, custom_colors)
    
    def _menu_activate_restore(self, menu, folders):
        """Restore Color"""
        for each_folder in folders:
            if each_folder.is_gone():
                continue
            
            folder_name = urllib.unquote(each_folder.get_uri()[7:])
            
            self.foldercolor.restore_icon(folder_name)
    
    def _menu_activate_emblem(self, menu, emblem, folders):
        """Click on emblem"""
        for each_folder in folders:
            if each_folder.is_gone():
                continue
            
            folder_name = urllib.unquote(each_folder.get_uri()[7:])
            new_emblem = self.foldercolor.get_iconname(folder_name, emblem, 'emblems')
            
            self.foldercolor.set_emblem(folder_name, new_emblem)
    
    def _menu_activate_donate(self, menu):
        """Click on donate"""
        webbrowser.open_new(self.foldercolor.URL_DONATE)
        # Hide donation after show it
        try:
            os.makedirs(self.foldercolor.HIDE_DONATION)
        except OSError as exception:
            pass
        except:
            pass
        return
